package com.cooksys.bootcamp.ballardFinal;

import javax.faces.context.FacesContext;
import javax.servlet.http.HttpSession;

import com.cooksys.bootcamp.ballardFinal.beans.UserBean;

// Generates the leading content for each page
public class PageFactory
{
	// Home page content
	private static String getHomePage()
	{
		return "<h1>Welcome!</h1><p/>Welcome to <i>Blakebuster</i>, your trusted online source for movie rentals and squids wearing tophats!<p/>"+
				"Please select a link from the menu on the left to get started.";
	}
	
	// Login page content
	private static String getLoginPage()
	{
		// Get the UserBean for this session
		FacesContext fcontext = FacesContext.getCurrentInstance();
		UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
		
		if (userBean.getLoggedIn())
		{
			// Already logged in. Get out here
			return "<h1>Login</h1><p/><b style='color:red;'>You are already logged in as " + userBean.getUsername() + "!</b>";
		}
		else
		{
			// Not logged in, so present the form here
			return "<h1>Login</h1>";
		}
	}
	
	// Logout page content
	private static String getLogoutPage()
	{
		// Get the UserBean for this session
		FacesContext fcontext = FacesContext.getCurrentInstance();
		UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
		
		if (userBean.getLoggedIn())
		{
			// Log out if we are logged in
			String userFirstName = userBean.getUserDetails().getFirstName();
			userBean.doLogout();
			
			return "<h1>Logout</h1><p/>Thanks for logging out, " + userFirstName + ". Come back soon!";
		}
		else
		{
			// This session was never logged in, so just show a message
			return "<h1>Logout</h1><p/>No user session found to close.";
		}
	}
	
	// Registration page content
	private static String getRegistrationPage()
	{
		// Get the UserBean for this session
		FacesContext fcontext = FacesContext.getCurrentInstance();
		UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
		
		if (userBean.getLoggedIn())
		{
			// Already logged in, so get out here
			return "<h1>Registration</h1><p/><b style='color:red;'>You are already logged in as " + userBean.getUsername() + "! Did you mean to edit your existing account?</b>";
		}
		else
		{
			// The rest of the content is handled by FormHandlerBean
			return "<h1>Registration</h1>";
		}
	}
	
	// Edit Account page content
	private static String getEditAccountPage()
	{
		// Get the UserBean for this session
		FacesContext fcontext = FacesContext.getCurrentInstance();
		UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
		
		if (userBean.getLoggedIn())
		{
			// The rest of the content is handled by FormHandlerBean
			return "<h1>Edit Account Details</h1>";
		}
		else
		{
			// Not logged in. Show an error and get out here
			return "<h1>Edit Account Details</h1><p/><b style='color:red;'>You must be logged in to edit details!</b>";
		}
	}
	
	// Rentals page
	private static String getRentalsPage()
	{
		// Get the UserBean for this session
		FacesContext fcontext = FacesContext.getCurrentInstance();
		UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
		
		if (userBean.getLoggedIn())
		{
			// The actual movie list is a form, so it is created by FormHandlerBean
			return "<h1>Rental History</h1><p/>Here is a list of movies you currently have rented out:";
		}
		else
		{
			// Not logged in, so show an error message
			return "<h1>Rental History</h1><p/><b style='color:red;'>You must be logged in to view your rental history!</b>";
		}
	}
	
	// Movies page
	private static String getMoviesPage()
	{
		// Get the UserBean for this session
		FacesContext fcontext = FacesContext.getCurrentInstance();
		UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
		
		if (userBean.getLoggedIn())
		{
			// The actual movie list is a form, so it is created by FormHandlerBean
			return "<h1>Movie List</h1><p/>Here is a list of the movies we currently have to offer";
		}
		else
		{
			// Not logged in, so show an information message about not being able to rent movies
			return "<h1>Movie List</h1><p/>Here is a list of the movies we currently have to offer<p/>"+
					"<b style='color:silver;'>(To rent a movie, you must first be logged in. If you don't have an account, registration is easy!)</b>";
		}
	}
	
	// A standard error page
	private static String getMissingPage()
	{
		return "Unknown page requested. Are you sure you're supposed to be here?";
	}
	
	// The entry point for this class. Returns the requested page
	public static String createPageContent(String page)
	{
		// page containts the name of the page we want to generate. Return the result of generating that page
		
		if (page.equals("home"))
		{
			return getHomePage();
		}
		else if (page.equals("login"))
		{
			return getLoginPage();
		}
		else if (page.equals("logout"))
		{
			return getLogoutPage();
		}
		else if (page.equals("register"))
		{
			return getRegistrationPage();
		}
		else if (page.equals("editacc"))
		{
			return getEditAccountPage();
		}
		else if (page.equals("rented"))
		{
			return getRentalsPage();
		}
		else if (page.equals("movies"))
		{
			return getMoviesPage();
		}
		else
		{
			// Unknown page type. Return the error page
			return getMissingPage();
		}
	}
}
