package com.cooksys.bootcamp.ballardFinal.beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.context.ApplicationContext;

import com.cooksys.bootcamp.ballardFinal.FormRow;
import com.cooksys.bootcamp.ballardFinal.MovieFilter;
import com.cooksys.prep.bo.AddressBo;
import com.cooksys.prep.bo.CategoryBo;
import com.cooksys.prep.bo.CityBo;
import com.cooksys.prep.bo.CountryBo;
import com.cooksys.prep.bo.FilmActorBo;
import com.cooksys.prep.bo.FilmBo;
import com.cooksys.prep.bo.FilmCategoryBo;
import com.cooksys.prep.bo.InventoryBo;
import com.cooksys.prep.bo.PaymentBo;
import com.cooksys.prep.bo.RentalBo;
import com.cooksys.prep.model.Address;
import com.cooksys.prep.model.Category;
import com.cooksys.prep.model.City;
import com.cooksys.prep.model.Country;
import com.cooksys.prep.model.Customer;
import com.cooksys.prep.model.Film;
import com.cooksys.prep.model.FilmActor;
import com.cooksys.prep.model.FilmCategory;
import com.cooksys.prep.model.Inventory;
import com.cooksys.prep.model.Payment;
import com.cooksys.prep.model.Rental;
import com.cooksys.prep.model.Staff;

// A backing bean for the input form on the site
@ManagedBean(name="formBean")
@ViewScoped
public class FormHandlerBean implements Serializable
{
	private static final long serialVersionUID = -6752704602719586642L;	// This class is serializable, so it requires a UID
	
	// Constants for the style attribute of the form and submit button
	private static final String FORM_DISPLAY_SHOW = "";
	private static final String FORM_DISPLAY_HIDE = "display:none;";
	
	private String formDisplay = FORM_DISPLAY_HIDE;			// Set the form to hidden by default
	private String formSubmitDisplay = FORM_DISPLAY_SHOW;	// Set the submit button to visible by default (may be hidden by the rest of the form)
	private FormRow[] formBody = {};		// Contains a list of table rows comprising the form
	
	// The movie filter for getting the list of films
	private MovieFilter movieFilter = new MovieFilter(null, "", "", "", 20, 0);
	
	// Default constructor
	public FormHandlerBean()
	{
		// We don't need to do anything. Variables have been set at creation time
	}
	
	public String getFormDisplay()
	{
		return formDisplay;
	}
	
	public void setFormDisplay(String formDisplay)
	{
		// Do nothing. This is managed internally
	}
	
	public String getFormSubmitDisplay()
	{
		return formSubmitDisplay;
	}
	
	public void setFormSubmitDisplay(String formSubmitDisplay)
	{
		// Do nothing. This is managed internally
	}
	
	public FormRow[] getFormBody()
	{
		return formBody;
	}
	
	public void setFormBody(FormRow[] formBody)
	{
		// Do nothing. This is managed internally
	}
	
	// Generate the film detail table for the given film
	private String renderFilmDetails(Film thisFilm)
	{
		// Set up our Spring resources for getting the inventory, shopping cart, and category list
		ApplicationContext appContext = PageBean.appContext;
    	FilmActorBo filmActorBo = (FilmActorBo)appContext.getBean("filmActorBo");
    	FilmCategoryBo filmCategoryBo = (FilmCategoryBo)appContext.getBean("filmCategoryBo");
    	
		// Get the actor list
		FilmActor[] filmActor = filmActorBo.findByFilm(thisFilm);
		StringBuffer actorList = new StringBuffer("<ul>");
		for (FilmActor thisActor : filmActor)
		{
			actorList.append("<li>" + thisActor.getActor().getFirstName() + " " + thisActor.getActor().getLastName() + "</li>");
		}
		actorList.append("</ul>");
		
		// Get the movie category list
		FilmCategory[] filmCategory = filmCategoryBo.findByFilm(thisFilm);
		StringBuffer movieCategoryList = new StringBuffer("<ul>");
		for (FilmCategory thisCategory : filmCategory)
		{
			movieCategoryList.append("<li>" + thisCategory.getCategory().getName() + "</li>");
		}
		movieCategoryList.append("</ul>");
		
		return "<div class='filmdetails' style='display:none;'><table>" +
	    			"<tr><td><b>Released:</b> " + thisFilm.getReleaseYear() + "</td><td><b>Rating:</b> " + thisFilm.getRating() + "</td>" +
	    			"<td><b>Length:</b> " + thisFilm.getLength() + "</td><td><b>Cost:</b> " + thisFilm.getRentalRate() + "</td></tr>" +
	    			"<tr><td colspan='2'><b>Actors:</b> <div>" + actorList.toString() + "</div></td><td colspan='2'><b>Categories:</b> <div>" + movieCategoryList.toString() + "</div></td></tr>" +
	    			"<tr><td colspan='4'>" + thisFilm.getDescription() + "</td></tr>" +
	    		"</table></div>";
	}
	
	// Generate the form content for the requested page
	public void renderFormForPage(String page)
	{
		// Show the submit button by default
		formSubmitDisplay = FORM_DISPLAY_SHOW;
		
		if (page.equals("login"))
		{
			// Create the login form
			formBody = new FormRow[]{new FormRow("<b>Username:</b>", "<input type='text' id='username' name='username' value='' maxlength='20' />"),
					new FormRow("<b>Password:</b>", "<input type='password' id='password' name='password' value='' />")};
			formDisplay = FORM_DISPLAY_SHOW;
		}
		else if (page.equals("register") || page.equals("editacc"))
		{
			// Set up the resources we need for accessing the current session
			FacesContext fcontext = FacesContext.getCurrentInstance();
			UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
			
			// Add the current values to the fields if this is the "Edit Account Details" page and the user is currently logged in
			Customer userDetails = new Customer();
			Address userAddress = new Address();
			int userCityId = -1;
			if (page.equals("editacc") && userBean.getLoggedIn())
			{
				userDetails = userBean.getUserDetails();
				userAddress = userDetails.getAddress();
				userCityId = userAddress.getCity().getCityId();
			}
			
			// Set up our Spring resources for getting the country list
			ApplicationContext appContext = PageBean.appContext;
	    	CountryBo countryBo = (CountryBo)appContext.getBean("countryBo");
	    	CityBo cityBo = (CityBo)appContext.getBean("cityBo");
			
			// Build the city list for each country
	    	Country[] countryList = countryBo.findAll();
	    	StringBuffer cityList = new StringBuffer("<select id='city' name='city'><option value='-1'" + (userCityId==-1?" selected ":"") + "></option>");
			for (Country thisCountry : countryList)
			{
				// Get the city list for this country
				City[] thisCityList = cityBo.findByCountry(thisCountry);
				
				// Add the country grouping
				cityList.append("<optgroup label='" + thisCountry.getCountry() + "'>");
				
				// Add the specific cities
				for (City thisCity : thisCityList)
				{
					cityList.append("<option value='" + thisCity.getCityId() + "'" + (userCityId==thisCity.getCityId()?" selected ":"") + ">" + thisCity.getCity() + "</option>");
				}
				cityList.append("</optgroup>");
			}
			cityList.append("</select>");
			
			// Create the register/edit form
			formBody = new FormRow[]{new FormRow("<b>Username:</b>", "<input type='text' id='username' name='username' value='" + userBean.getUsername() + "' maxlength='20' />"),
					new FormRow("<b>Password:</b>", "<input type='password' id='password' name='password' value='' />"),
					new FormRow((page.equals("editacc")?"<b>New Password:</b> (optional)":"<b>Confirm Password:</b>"), "<input type='password' id='password2' name='password2' value='' />"),
					new FormRow("<b>First Name:</b>", "<input type='text' id='first_name' name='first_name' value='" + (userDetails.getFirstName()==null?"":userDetails.getFirstName()) + "' maxlength='45' />"),
					new FormRow("<b>Last Name:</b>", "<input type='text' id='last_name' name='last_name' value='" + (userDetails.getLastName()==null?"":userDetails.getLastName()) + "' maxlength='45' />"),
					new FormRow("<b>Email:</b> (optional)", "<input type='text' id='email' name='email' value='" + (userDetails.getEmail()==null?"":userDetails.getEmail()) + "' maxlength='50' />"),
					new FormRow("<b>Address:</b> (optional)", "<input type='text' id='address' name='address' value='" + (userAddress.getAddress()==null?"":userAddress.getAddress()) + "' maxlength='50' />"),
					new FormRow("<b>City:</b>", cityList.toString()),
					new FormRow("<b>State/District:</b>", "<input type='text' id='district' name='district' value='" + (userAddress.getDistrict()==null?"":userAddress.getDistrict()) + "' maxlength='20' />"),
					new FormRow("<b>Zip Code:</b> (optional)", "<input type='text' id='zip' name='zip' value='" + (userAddress.getPostalCode()==null?"":userAddress.getPostalCode()) + "' maxlength='10' />"),
					new FormRow("<b>Phone:</b> (optional)", "<input type='text' id='phone' name='phone' value='" + (userAddress.getPhone()==null?"":userAddress.getPhone()) + "' maxlength='20' />")};
			formDisplay = FORM_DISPLAY_SHOW;
		}
		else if (page.equals("rented"))
		{
			// Set up the resources we need for accessing the current session
			FacesContext fcontext = FacesContext.getCurrentInstance();
			UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
			
			if (userBean.getLoggedIn())
			{
				// Logged in. Generate the rental history form
				
				// Set up our Spring resources for getting the rented movies
				ApplicationContext appContext = PageBean.appContext;
		    	RentalBo rentalBo = (RentalBo)appContext.getBean("rentalBo");
		    	
		    	// Get the list of movies this user currently has rented out
		    	Rental[] rentals = rentalBo.findDueByCustomer(userBean.getUserDetails());
		    	ArrayList<FormRow> displayList = new ArrayList<FormRow>();
		    	
		    	// Display each rental and a return button
		    	for (int i = 0; i < rentals.length; i++)
		    	{
		    		Film thisFilm = rentals[i].getInventory().getFilm();
		    		displayList.add(new FormRow("<a href='#' onclick='setSelectionAndSubmit(\"return\", " + rentals[i].getRentalId() + ");return false;'>Return</a>",
		    						"<table style='min-width:400px;'>" +
		    						"<tr><td><a href='#' onclick='toggleMovieDetails($(this));return false;'>&nbsp;&gt;&nbsp;</a>&nbsp;<span class='filmtitle'>" + thisFilm.getTitle() + "</span></td></tr>" +
		    						"<tr><td>" + renderFilmDetails(thisFilm) + "</td></tr>" +
		    						"</table>"));
		    	}
		    	
		    	// If the user doesnt have any rentals, show a message
		    	if (rentals.length == 0)
		    	{
		    		displayList.add(new FormRow("", "<b>You have no rentals out</b>"));
		    	}
		    	
		    	// Show the form, but hide the submit button because this form doesn't require submission
		    	formBody = displayList.toArray(new FormRow[0]);
		    	formSubmitDisplay = FORM_DISPLAY_HIDE;
				formDisplay = FORM_DISPLAY_SHOW;
			}
			else
			{
				// Not logged in, so don't waste our time trying to generate a form
				formBody = new FormRow[]{};
				formDisplay = FORM_DISPLAY_HIDE;
			}
		}
		else if (page.equals("movies"))
		{
			// Set up the resources we need for accessing the current session
			FacesContext fcontext = FacesContext.getCurrentInstance();
			UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
			
			// Set up our Spring resources for getting the inventory, shopping cart, and category list
			ApplicationContext appContext = PageBean.appContext;
			InventoryBo inventoryBo = (InventoryBo)appContext.getBean("inventoryBo");
			FilmBo filmBo = (FilmBo)appContext.getBean("filmBo");
	    	RentalBo rentalBo = (RentalBo)appContext.getBean("rentalBo");
	    	CategoryBo categoryBo = (CategoryBo)appContext.getBean("categoryBo");
	    	
	    	// Find the movies that are within the given criteria
	    	Film[] rentable = filmBo.findAll(movieFilter);
	    	Rental[] shoppingcart = new Rental[0];
	    	
	    	// Use this to temporarily hold the page form data
	    	ArrayList<FormRow> displayList = new ArrayList<FormRow>();
	    	
	    	// Build the category list for the search form
	    	Category[] categories = categoryBo.findAll();
	    	StringBuffer categoryList = new StringBuffer("<select id='category' name='category'><option value='-1'" + (movieFilter.category==null?" selected ":"") + ">All</option>");
	    	for (Category thisCategory : categories)
	    	{
	    		categoryList.append("<option value='" + thisCategory.getCategoryId() + "'" + (movieFilter.category==thisCategory?" selected ":"") + ">" + thisCategory.getName() + "</option>");
	    	}
	    	categoryList.append("</select>");
	    	
	    	// Display the search form
	    	displayList.add(new FormRow("<b>Filter movies by:</b>", 
	    			"<table border=1>" +
	    			"<tr>" +
	    				"<td>Category:</td><td>" + categoryList.toString() + "</td>" +
	    				"<td>Title:</td><td><input type='text' id='title' name='title' value='" + movieFilter.title.replaceAll("\'", "\\\'") + "' /></td>" +
	    			"</tr><tr>" +
	    				"<td>Actor first name:</td><td><input type='text' id='actorFirst' name='actorFirst' value='" + movieFilter.actorFirst.replaceAll("\'", "\\\'") + "' /></td>" +
	    				"<td>Actor last name:</td><td><input type='text' id='actorLast' name='actorLast' value='" + movieFilter.actorLast.replaceAll("\'", "\\\'") + "' /></td>" +
	    			"</tr><tr>" +
	    				"<td colspan=4><center><a href='#' onclick='setSelectionAndSubmit(\"filter\", \"\");return false;'>Go</a></center></td>" +
	    			"</tr></table>"));
	    	
	    	// Display the page navigation buttons
	    	displayList.add(new FormRow("", (movieFilter.pageNumber>0?"<a href='#' onclick='setSelectionAndSubmit(\"pageover\", \"prev\");return false;'>&lt;&nbsp;Previous page</a>":"") + "<a href='#' onclick='setSelectionAndSubmit(\"pageover\", \"next\");return false;' style='float:right;'>Next page&nbsp;&gt;</a>"));
	    	
	    	// Display the films and "add to cart" buttons
	    	int displaySize = displayList.size();
	    	for (Film thisFilm : rentable)
	    	{
	    		// Get the inventories
	    		thisFilm.setInventories(new HashSet<Inventory>( Arrays.asList(inventoryBo.findAllRentable(thisFilm)) ));
	    		
	    		// Create the movie list entry
	    		displayList.add(new FormRow((userBean.getLoggedIn()&&thisFilm.getInventories().size()>0?"<a href='#' onclick='setSelectionAndSubmit(\"addcart\", " + thisFilm.getFilmId() + ");return false;'>Add to cart</a>":"&nbsp;"), 
	    					"<table style='min-width:400px;'>" +
	    					"<tr><td><a href='#' onclick='toggleMovieDetails($(this));return false;'>&nbsp;&gt;&nbsp;</a>&nbsp;<span class='filmtitle'>" + thisFilm.getTitle() + "</span>&nbsp;<i>(" + thisFilm.getInventories().size() + " in stock)</i></td></tr>" +
	    					"<tr><td>" + renderFilmDetails(thisFilm) + "</td></tr>" +
	    					"</table>"));
	    	}
	    	
	    	// If no movies are displayed, show the user a message
	    	if (displayList.size() == displaySize)
	    	{
	    		displayList.add(new FormRow("&nbsp;", "<b>No movies match your criteria</b>"));
	    	}
	    	
	    	// If the user is logged in, show the shopping cart list and "Checkout" submit button
	    	if (userBean.getLoggedIn())
	    	{
	    		shoppingcart = rentalBo.findReservedByCustomer(userBean.getUserDetails());
		    	displayList.add(new FormRow("<b>Shopping Cart:</b>", ""));
		    	
		    	// Create the movie list for the shopping cart
		    	for (int i = 0; i < shoppingcart.length; i++)
		    	{
		    		Film thisFilm = shoppingcart[i].getInventory().getFilm();
		    		displayList.add(new FormRow("<a href='#' onclick='setSelectionAndSubmit(\"removecart\", " + shoppingcart[i].getRentalId() + ");return false;'>Remove from cart</a>",
		    				"<table style='min-width:400px;'>" +
		    				"<tr><td><a href='#' onclick='toggleMovieDetails($(this));return false;'>&nbsp;&gt;&nbsp;</a>&nbsp;<span class='filmtitle'>" + thisFilm.getTitle() + "</span></td></tr>" +
		    				"<tr><td>" + renderFilmDetails(thisFilm) + "</td></tr>" +
		    				"</table>"));
		    	}
	    	}
	    	else
	    	{
	    		formSubmitDisplay = FORM_DISPLAY_HIDE;
	    	}
	    	
	    	formBody = displayList.toArray(new FormRow[0]);
			formDisplay = FORM_DISPLAY_SHOW;
		}
		else
		{
			// The requested page doesn't require a form, so hide it
			formBody = new FormRow[]{};
			formDisplay = FORM_DISPLAY_HIDE;
		}
	}
	
	// Form submission logic for the requested page
	public String handleFormSubmit(String page)
	{
		boolean success;	// Track the success of the form submission
		String result;		// An optional error/success message to show on the content lead
		
		// Set up the resources we need for accessing the submitted form data
		FacesContext fcontext = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest)fcontext.getExternalContext().getRequest();
		
		if (page.equals("login"))
		{
			// Login logic
			
			// Get the UserBean for this session
			UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
			
			// Get the submitted form data
			String username = request.getParameter("username");
			String password = request.getParameter("password");
			
			// Attempt to login with the provided username and password
			String loginMessage = userBean.doLogin(username, password);
			
			if (userBean.getLoggedIn())
			{
				// Login was successful
				success = true;
				result = "<b style='color:green;'>Logged in successfully! " + loginMessage + "</b>";
			}
			else
			{
				// Login failed
				success = false;
				result = "<b style='color:red;'>Login failed: " + loginMessage + "</b>";
			}
		}
		else if (page.equals("register") || page.equals("editacc"))
		{
			// The Registration and Edit Account Details forms contain the same information, so we've merged the logic here
			
			// Get the UserBean for this session
			UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
			
			// Get the submitted form data
			String username = request.getParameter("username");
			String password = request.getParameter("password");
			String password2 = request.getParameter("password2");
			String firstName = request.getParameter("first_name");
			String lastName = request.getParameter("last_name");
			String email = request.getParameter("email");
			String address = request.getParameter("address");
			short cityId = Short.parseShort(request.getParameter("city"));
			String district = request.getParameter("district");
			String zip = request.getParameter("zip");
			String phone = request.getParameter("phone");
			
			// Check to make sure required fields were filled out
			if (username.isEmpty() || password.isEmpty())
			{
				// No username or password
				success = false;
				result = "<b style='color:red;'>Username and password must be provided!</b>";
			}
			else if (firstName.isEmpty() || lastName.isEmpty())
			{
				// No first or last name
				success = false;
				result = "<b style='color:red;'>First and last name must be provided!</b>";
			}
			else if (cityId == -1 || district.isEmpty())
			{
				// No city or state/district
				success = false;
				result = "<b style='color:red;'>Please select a city and provide a state/district</b>";
			}
			// Required fields were filled out, so run the necessary page logic
			else if (page.equals("register"))
			{
				// Registration logic
				
				Object[] updateResult;
				
				// Check to make sure the password and confirmation password fields matched
				if (!password.equals(password2))
				{
					updateResult = new Object[]{(Boolean)false, "The passwords you provided did not match!"};
				}
				else
				{
					// Set up our Spring resources for creating the address for the new customer
					ApplicationContext appContext = PageBean.appContext;
			    	AddressBo addressBo = (AddressBo)appContext.getBean("addressBo");
			    	
			    	// Create the address object
			    	City city = new City();
			    	city.setCityId(cityId);
			    	Address addressObject = new Address(city, address, district, phone, new Date());
			    	
			    	// Save the address
			    	addressBo.save(addressObject);
			    	
			    	// Register the rest of the user's information
					updateResult = userBean.doRegistration(username, password, firstName, lastName, email, addressObject);
				}
				
				// Return a result message depending on if the registration was successful or not
				if (((Boolean)updateResult[0]).booleanValue() == true)
				{
					success = true;
					result = "<b style='color:green;'>Registered and logged in as " + username + "!</b>";
				}
				else
				{
					success = false;
					result = "<b style='color:red;'>" + (String)updateResult[1] + "</b>";
				}
			}
			else
			{
				// Edit Account Details logic
				
				if (!userBean.getLoggedIn())
				{
					// Not logged in. We can't edit an account if the user isn't logged in
					success = false;
					result = "<b style='color:red;'>You must be logged in to edit an account!</b>";
				}
				else
				{
					// Update the credentials (username and password). This will also confirm that the user provided the correct current password
					Object[] updateResult = userBean.doUpdateCredentials(username, password, password2);
					
					// If there was a problem updating credentials (bad information or an incorrect password), skip updating details
					if (((Boolean)updateResult[0]).booleanValue() == true)
					{
						// Set up our Spring resources for updating the address
						ApplicationContext appContext = PageBean.appContext;
				    	AddressBo addressBo = (AddressBo)appContext.getBean("addressBo");
				    	
				    	// Update the address
				    	Address addressObject = userBean.getUserDetails().getAddress();
				    	City city = new City();
				    	city.setCityId(cityId);
				    	addressObject.setAddress(address);
				    	addressObject.setCity(city);
				    	addressObject.setDistrict(district);
				    	addressObject.setPhone(phone);
				    	addressObject.setPostalCode(zip);
				    	addressObject.setLastUpdate(new Date());
				    	addressBo.update(addressObject);
				    	userBean.getUserDetails().setAddress(addressObject);
						
						// Update user details
						updateResult = userBean.doUpdateUserDetails(firstName, lastName, email, addressObject);
					}
					
					// Return a result message depending on if the update was successful or not
					if (((Boolean)updateResult[0]).booleanValue() == true)
					{
						success = true;
						result = "<b style='color:green;'>Profile successfully updated!</b>";
					}
					else
					{
						success = false;
						result = "<b style='color:red;'>" + (String)updateResult[1] + "</b>";
					}
				}
			}
		}
		else if (page.equals("rented"))
		{
			// Get the submitted form data
			String submissionType = request.getParameter("formBeanSubmissionType");
			String submissionValue = request.getParameter("formBeanSubmissionValue");
			
			if (submissionType.equals("return"))
			{
				// Return the rental specified in submissionValue
				
				// Set up our Spring resources for updating the rental
				ApplicationContext appContext = PageBean.appContext;
		    	RentalBo rentalBo = (RentalBo)appContext.getBean("rentalBo");
		    	
		    	// Get the rental we want to update (submissionValue contains the rental ID)
		    	Rental thisRental = rentalBo.findById(Short.parseShort(submissionValue));
		    	
		    	// Set the return date and update
		    	thisRental.setReturnDate(new Date());
		    	rentalBo.update(thisRental);
		    	
		    	// Rebuild the page to show the changes
		    	renderFormForPage(page);
				
		    	// Say the submission failed in order to keep the form visible
				success = false;
				result = "";
			}
			else
			{
				success = false;
				result = "<b style='color:red;'>Unknown submission type</b>";
			}
		}
		else if (page.equals("movies"))
		{
			// Get the submitted form data
			String submissionType = request.getParameter("formBeanSubmissionType");
			String submissionValue = request.getParameter("formBeanSubmissionValue");
			
			// Get the UserBean for this session
			UserBean userBean = (UserBean)((HttpSession)fcontext.getExternalContext().getSession(true)).getAttribute("userBean");
			
			if (submissionType.isEmpty())
			{
				// Checkout logic
				
				// Set up our Spring resources for updating the rental and payment info
				ApplicationContext appContext = PageBean.appContext;
		    	RentalBo rentalBo = (RentalBo)appContext.getBean("rentalBo");
		    	PaymentBo paymentBo = (PaymentBo)appContext.getBean("paymentBo");
		    	
		    	// Get the rentals we want to check out
		    	Rental[] reservedList = rentalBo.findReservedByCustomer(userBean.getUserDetails());
		    	
		    	// Pay for the rental
		    	for (Rental thisRental : reservedList)
		    	{
		    		// Use the dummy staff
		    		Staff thisStaff = new Staff();
		    		thisStaff.setStaffId(new Byte((byte)1));
		    		
		    		// Create and save the payment
		    		Payment payment = new Payment(thisStaff, userBean.getUserDetails(), thisRental, thisRental.getInventory().getFilm().getRentalRate(), new Date(), new Date());
		    		paymentBo.save(payment);
		    	}
				
				success = true;
				result = "<b style='color:green;'>Checked out movies! Return movies on the Rentals page</b>";
			}
			else if (submissionType.equals("filter"))
			{
				// Filter logic
				
				// Get the search form
				short categoryId = Short.parseShort(request.getParameter("category"));
				String title = request.getParameter("title");
				String actorFirst = request.getParameter("actorFirst");
				String actorLast = request.getParameter("actorLast");
				
				// Set up our Spring resources for getting the category
				ApplicationContext appContext = PageBean.appContext;
				CategoryBo categoryBo = (CategoryBo)appContext.getBean("categoryBo");
				
				// Get the category
				Category category = categoryBo.findById(categoryId);
				
				// Save the new filter
				movieFilter.category = category;
				movieFilter.title = title;
				movieFilter.actorFirst = actorFirst;
				movieFilter.actorLast = actorLast;
				movieFilter.pageNumber = 0;
				
				// Rebuild the page to show the changes
		    	renderFormForPage(page);
		    	
		    	// Say the submission failed in order to keep the form visible
		    	success = false;
		    	result = "";
			}
			else if (submissionType.equals("pageover"))
			{
				// Page Navigation logic
				
				if (submissionValue.equals("prev"))
				{
					// Go back a page
					if (movieFilter.pageNumber > 0) movieFilter.pageNumber--;
				}
				else
				{
					// Go forward a page
					movieFilter.pageNumber++;
				}
				
				// Rebuild the page to show the changes
		    	renderFormForPage(page);
		    	
		    	// Say the submission failed in order to keep the form visible
		    	success = false;
		    	result = "";
			}
			else if (submissionType.equals("addcart"))
			{
				// Add-to-cart logic
				
				// Set up our Spring resources for creating the rental
				ApplicationContext appContext = PageBean.appContext;
				FilmBo filmBo = (FilmBo)appContext.getBean("filmBo");
				InventoryBo inventoryBo = (InventoryBo)appContext.getBean("inventoryBo");
		    	RentalBo rentalBo = (RentalBo)appContext.getBean("rentalBo");
		    	
		    	// Find available inventory for us to rent (submissionValue contains the film ID we want to rent)
		    	Inventory[] inventoryAvailable = inventoryBo.findAllRentable(filmBo.findById(Short.parseShort(submissionValue)));
		    	
		    	if (inventoryAvailable.length > 0)
		    	{
		    		// The requested movie is in stock
		    		
		    		// Use the dummy staff
		    		Staff thisStaff = new Staff();
		    		thisStaff.setStaffId(new Byte((byte)1));
		    		
		    		// Create the rental (we could use any of the available inventories)
		    		Rental thisRental = new Rental(thisStaff, userBean.getUserDetails(), inventoryAvailable[0], new Date(), new Date());
		    		rentalBo.save(thisRental);
		    		
		    		// Say the submission failed in order to keep the form visible
		    		success = false;
		    		result = "";
		    	}
		    	else
		    	{
		    		// The requested movie isn't in stock
		    		success = false;
		    		result = "<script>alert('Oops! That movie is now out of stock!')</script>";
		    	}
		    	
		    	// Rebuild the page to show the changes
		    	renderFormForPage(page);
			}
			else if (submissionType.equals("removecart"))
			{
				// Remove-from-cart logic
				
				// Set up our Spring resources for deleting the rental
				ApplicationContext appContext = PageBean.appContext;
				RentalBo rentalBo = (RentalBo)appContext.getBean("rentalBo");
				
				// Get the rental we want to delete (submissionValue contains the rental ID we want to delete)
				Rental thisRental = rentalBo.findById(Short.parseShort(submissionValue));
				
				if (thisRental != null && thisRental.getCustomer().getCustomerId().shortValue() == userBean.getUserDetails().getCustomerId().shortValue())
				{
					// Delete the rental
					rentalBo.delete(thisRental);
					
					// Say the submission failed in order to keep the form visible
					success = false;
					result = "";
				}
				else
				{
					// Either the rental has already been deleted or this customer isnt the owner of it
					success = false;
					result = "<script>alert('Oops! That rental isnt in your cart!')</script>";
				}
				
		    	// Rebuild the page to show the changes
		    	renderFormForPage(page);
			}
			else
			{
				success = false;
				result = "<b style='color:red;'>Unknown submission type</b>";
			}
		}
		else
		{
			// Somehow, a form got submitted for a page that isn't supposed to contain a form
			success = true;
			result = "<b style='color:red;'>Unknown form type</b>";
		}
		
		// If the form submitted successfully, hide the form on the resulting page
		if (success)
		{
			formBody = new FormRow[]{};
			formDisplay = FORM_DISPLAY_HIDE;
		}
		
		// Return any error/success messages created during form submission
		return result;
	}
}
