package com.cooksys.bootcamp.ballardFinal.beans;

import java.io.Serializable;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;

import org.icefaces.application.PushRenderer;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.cooksys.bootcamp.ballardFinal.PageFactory;

// This is the main backing bean. It contains all other beans and controls the ICEpush session
@ManagedBean(name="pageBean")
@ViewScoped
public class PageBean implements Serializable
{
	private static final long serialVersionUID = -7509420961117348069L;		// This class is serializable, so it requires a UID
	private static final String PUSH_GROUP = "myPage";						// A push group for the ICEpush session
	
	// A public-accessible reference to the Spring context. This means we only ever have to load Spring once per view
	public static final ApplicationContext appContext = new ClassPathXmlApplicationContext("springconfig/context/app-context.xml");
	
	@ManagedProperty(value="#{userBean}")
	private UserBean userBean = new UserBean();					// The user session bean
	@ManagedProperty(value="#{formBean}")
	private FormHandlerBean formBean = new FormHandlerBean();	// The form generating bean
	
	private String selectedPage;	// What page was requested?
	private String pageContent;		// What content do we need to push back to the page?
	
	public PageBean()
	{
		// Set up ICEpush for the session
		PushRenderer.addCurrentSession(PUSH_GROUP);
		
		// By default, we start on the home page
		setSelectedPage("home");
		choosePage();
	}
	
	public UserBean getUserBean()
	{
		return userBean;
	}
	
	public void setUserBean(UserBean userBean)
	{
		this.userBean = userBean;
	}
	
	public FormHandlerBean getFormBean()
	{
		return formBean;
	}
	
	public void setFormBean(FormHandlerBean formBean)
	{
		this.formBean = formBean;
	}
	
	public String getSelectedPage()
	{
		return selectedPage;
	}
	
	public void setSelectedPage(String selectedPage)
	{
		this.selectedPage = selectedPage;
	}
	
	// After the page is selected (setSelectedPage), generate the content and push it back
	public String choosePage()
	{
		pageContent = PageFactory.createPageContent(selectedPage);	// Generate the leading content
		formBean.renderFormForPage(selectedPage);					// Generate the form for the page (if one is necessary)
		
		// Tell ICEpush to render the new content
		PushRenderer.render(PUSH_GROUP);
		
		// ICEfaces expects a return value from AJAX-callable methods. Just return null to make it happy
		return null;
	}
	
	public String getPageContent()
	{
		return pageContent;
	}
	
	public void setPageContent(String pageContent)
	{
		// Do nothing. This bean manages page content internally
	}
	
	// A redirecting function for FormHandlerBean's handleFormSubmit function
	public String submitForm()
	{
		// Generate the leading content PLUS any error/success messages from the form bean
		pageContent = PageFactory.createPageContent(selectedPage) + "<p/>" + formBean.handleFormSubmit(selectedPage);
		
		// Tell ICEpush to render the new content
		PushRenderer.render(PUSH_GROUP);
		
		// ICEfaces expects a return value from AJAX-callable methods. Just return null to make it happy
		return null;
	}
}
