package com.cooksys.bootcamp.ballardFinal.beans;

import java.io.Serializable;
import java.security.MessageDigest;
import java.util.Date;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;

import org.springframework.context.ApplicationContext;

import com.cooksys.prep.bo.CustomerBo;
import com.cooksys.prep.bo.CustomerLoginBo;
import com.cooksys.prep.model.Address;
import com.cooksys.prep.model.Customer;
import com.cooksys.prep.model.CustomerLogin;
import com.cooksys.prep.model.Store;

// A bean to contain and manage the user's session
@ManagedBean(name="userBean")
@SessionScoped
public class UserBean implements Serializable
{
	private static final long serialVersionUID = 7412078967266333033L;	// This class is serializable, so it requires a UID
	
	private String username = "";			// The username this session is logged in with
	private Customer userDetails = null;	// The customer details for this session
	
	// Default constructor
	public UserBean()
	{
		// Do nothing. Our variables are set at creation time
	}
	
	public String getUsername()
	{
		return username;
	}
	
	public void setUsername(String username)
	{
		this.username = username;
	}
	
	public Customer getUserDetails()
	{
		return userDetails;
	}
	
	public void setUserDetails(Customer userDetails)
	{
		this.userDetails = userDetails;
	}
	
	public boolean getLoggedIn()
	{
		if (userDetails == null) return false;
		else return true;
	}
	
	public void setLoggedIn()
	{
		// Do nothing. This is managed internally
	}
	
	// Attempt to login using the provided username and password
	public String doLogin(String username, String password)
	{
		// Set up our Spring resources for the business objects
		ApplicationContext appContext = PageBean.appContext;
    	CustomerLoginBo customerLoginBo = (CustomerLoginBo)appContext.getBean("customerLoginBo");
    	CustomerBo customerBo = (CustomerBo)appContext.getBean("customerBo");
    	
    	// Find the customer information for the given credentials
    	CustomerLogin customerLogin = customerLoginBo.findByCredentials(username, password);
    	if (customerLogin == null)
    	{
    		return "Username/password mismatch";
    	}
    	
    	// Retrieve the customer object for this login
    	Customer myCustomer = customerBo.findById(customerLogin.getCustomerId());
    	if (myCustomer == null)
    	{
    		return "No customer associated with that login";
    	}
    	
    	// Update the customer's last login time
    	Date lastLogin = customerLogin.getLastLogin();
    	customerLogin.setLastLogin(new Date());
    	customerLoginBo.update(customerLogin);
    	
    	// Update this UserBean to reflect the successful login
    	setUsername(username);
		setUserDetails(myCustomer);
		return "Welcome " + userDetails.getFirstName() + " (last logged in " + lastLogin.toString() + ")";
	}
	
	// Logout of this user session
	public void doLogout()
	{
		// Just reset the username and customer object. The database doesn't need anything extra
		setUsername("");
		setUserDetails(null);
	}
	
	// Create a customer and login entry with the given details, then attempt to login
	public Object[] doRegistration(String username, String password, String firstName, String lastName, String email, Address address)
	{
		// If the username or password are blank, stop here
		if (username.isEmpty() || password.isEmpty())
		{
			return new Object[]{(Boolean)false, "Username and password are required"};
		}
		
		// Set up our Spring resources for the business objects
		ApplicationContext appContext = PageBean.appContext;
    	CustomerLoginBo customerLoginBo = (CustomerLoginBo)appContext.getBean("customerLoginBo");
    	CustomerBo customerBo = (CustomerBo)appContext.getBean("customerBo");
    	
    	// Create our message digest object for hashing the password
    	MessageDigest md;
		try
		{
			md = MessageDigest.getInstance("MD5");
		}
		catch (Exception e)
		{
			return new Object[]{(Boolean)false, "Could not generate hashing digest"};
		}
		
		// Generate the hash for the new password
    	md.update(password.getBytes());
    	byte byteData[] = md.digest();
    	
    	// Format the hash as a String
    	StringBuffer sb = new StringBuffer();
    	for (int i = 0; i < byteData.length; i++)
    	{
    		String hex = Integer.toHexString(0xFF & byteData[i]);
    		sb.append((hex.length()==1?"0":"") + hex);
    	}
    	String passwordHash = sb.toString();
    	
    	// Create the new customer (using the default store)
    	Store store = new Store();
    	store.setStoreId((byte)1);
    	Customer customer = new Customer(address, store, firstName, lastName, email, true, new Date(), new Date());
    	customerBo.save(customer);
    	
    	// Create the new login credentials
    	CustomerLogin customerLogin = new CustomerLogin(customer.getCustomerId(), username, passwordHash, new Date());
    	customerLoginBo.save(customerLogin);
    	
    	// Log in as the new customer
    	String loginResult = doLogin(username, password);
    	if (!getLoggedIn())
    	{
    		return new Object[]{(Boolean)true, "Registration completed, but could not log in: " + loginResult};
    	}
    	
    	return new Object[]{(Boolean)true};
	}
	
	// Update the username and password of the currently logged in user
	public Object[] doUpdateCredentials(String newUsername, String oldPassword, String newPassword)
	{
		// Set up our Spring resources for the business object
		ApplicationContext appContext = PageBean.appContext;
    	CustomerLoginBo customerLoginBo = (CustomerLoginBo)appContext.getBean("customerLoginBo");
    	
    	// Find the customer information for the given credentials
    	CustomerLogin customerLogin = customerLoginBo.findByCredentials(getUsername(), oldPassword);
    	
    	// If there is no matching username-password combination in the database, return an error
    	if (customerLogin == null)
    	{
    		return new Object[]{(Boolean)false, "Incorrect password"};
    	}
    	
    	// Update the username if a new one was provided
    	if (newUsername.length() > 0) customerLogin.setUsername(newUsername);
    	
    	// Update the password if a new one was provided
    	if (newPassword.length() > 0 && !newPassword.equals(oldPassword))
    	{
	    	// Create our message digest object for hashing the password
	    	MessageDigest md;
			try
			{
				md = MessageDigest.getInstance("MD5");
			}
			catch (Exception e)
			{
				return new Object[]{(Boolean)false, "Could not generate hashing digest"};
			}
			
			// Generate the hash for the new password
	    	md.update(newPassword.getBytes());
	    	byte byteData[] = md.digest();
	    	
	    	// Format the hash as a String
	    	StringBuffer sb = new StringBuffer();
	    	for (int i = 0; i < byteData.length; i++)
	    	{
	    		String hex = Integer.toHexString(0xFF & byteData[i]);
	    		sb.append((hex.length()==1?"0":"") + hex);
	    	}
	    	String passwordHash = sb.toString();
	    	
	    	// Update the password
	    	customerLogin.setPassword(passwordHash);
    	}
    	
    	// Save the new user credentials
    	customerLoginBo.update(customerLogin);
    	setUsername(customerLogin.getUsername());
    	
		return new Object[]{(Boolean)true};
	}
	
	// Update the customer information
	public Object[] doUpdateUserDetails(String firstName, String lastName, String email, Address address)
	{
		// Set up our Spring resources for the business objects
		ApplicationContext appContext = PageBean.appContext;
		CustomerBo customerBo = (CustomerBo)appContext.getBean("customerBo");
    	
    	// Update and save the user details
		userDetails.setFirstName(firstName);
    	userDetails.setLastName(lastName);
    	userDetails.setEmail(email);
    	userDetails.setAddress(address);
    	userDetails.setLastUpdate(new Date());
    	customerBo.update(userDetails);
    	
    	return new Object[]{(Boolean)true};
	}
}